import os

#normal job, no checkpoint
def slurm_prepare_job_script_python(iStart,  iEnd,  sBasename_checkpoint, sBasename_job, sBasename_python, sDirectory_job, sDirectory_python, sJob_name, iFlag_resubmit =None,  iWalltime_in = None, nNode_in = None, nTask_in=None, sAccount = None, sEmail=None, sQueue_in=None):
    """
    Prepare a slurm job script for python with checkpoint

    Args:
        iStart (int): Checkpoint start index
        iEnd (int): Checkpoint end index
        sBasename_checkpoint (string): Basename 
        sBasename_job (string): Job name
        sBasename_python (string): Python path
        sDirectory_job (string): The path where contains the bash script
        sDirectory_python (string): The folder of Python
        sJob_name (string): Job name
        iFlag_resubmit (int, optional): Flag to indicate this is a resubmit. Defaults to None.
        iWalltime_in (int, optional): The walltime in hour. Defaults to None.
        nNode_in (int, optional): Number of node. Defaults to None.
        nTask_in (int, optional): Number of task. Defaults to None.
        sAccount (string, optional): Cluster account. Defaults to None.
        sEmail (string, optional): Email address. Defaults to None.
        sQueue_in (string, optional): Cluster partition. Defaults to None.
    """

    if nNode_in is not None:
        iNode = nNode_in
    else:
        iNode = 1

    if nTask_in is not None:
        nTask = nTask_in
    else:
        nTask = 10



    if sQueue_in is not None:
        sQueue = sQueue_in
    else:
        sQueue = 'short'

    if iWalltime_in is not None:
        iWalltime = iWalltime_in
    else:
        iWalltime = 2

    sStart = "{:0d}".format(iStart  )
    sEnd =  "{:0d}".format(iEnd  )
    sNode =  "{:0d}".format(iNode  )
    sTask =   "{:0d}".format(nTask  )
    sWalltime ="{:0d}".format(iWalltime  )
    os.chdir(sDirectory_job)

    pFile =  open(sBasename_job,"w")  #write mode
    sLine = '#!/bin/bash' + '\n'
    pFile.write( sLine )

    if sAccount is not None:
        sLine = '#SBATCH --account=' + sAccount  + '\n'
        pFile.write( sLine )

    sLine = '#SBATCH --begin=now+1minutes' + '\n'
    pFile.write( sLine )

    sLine = '#SBATCH --cpus-per-task=1 ' + '\n'
    pFile.write( sLine )

    sLine = '#SBATCH --dependency=singleton ' + '\n'
    pFile.write( sLine )
    sLine = '#SBATCH --error=stderr_%j.err' + '\n'
    pFile.write( sLine )

    if sJob_name is not None:
        sLine = '#SBATCH --job-name=' + sJob_name + '  # create a name for your job' + '\n'
        pFile.write( sLine )

    sLine = '#SBATCH --mail-type=ALL' + '\n'
    pFile.write( sLine )

    if sEmail is not None:
        sLine = '#SBATCH --mail-user=' + sEmail + '\n'
        pFile.write( sLine )

    sLine = '#SBATCH --nodes=' + sNode + ' # node count' + '\n'
    pFile.write( sLine )
    sLine = '#SBATCH --ntasks=' + sTask + ' # total number of tasks' + '\n'
    pFile.write( sLine )

    sLine = '#SBATCH --output=stdout_%j.out' + '\n'
    pFile.write( sLine )

    sLine = '#SBATCH --partition=' + sQueue + '\n'  #can be improved here
    pFile.write( sLine )

    sLine = '#SBATCH --time=' + sWalltime +':00:00      # total run time limit (HH:MM:SS)' + '\n'
    pFile.write( sLine )

    sLine = 'module purge' + '\n'
    pFile.write( sLine )
    sLine = 'module load anaconda3/2019.03' + '\n'
    pFile.write( sLine )
    #only need to manuually source if the cluster requires
    #sLine = 'source /share/apps/anaconda3/2019.03/etc/profile.d/conda.sh' + '\n'
    #pFile.write( sLine )
    sLine = 'unset PYTHONHOME' + '\n'
    pFile.write( sLine )
    sLine = 'conda activate mpienv' + '\n'
    pFile.write( sLine )
    sLine = 'sDirectory_job=' + sDirectory_job + '\n'
    pFile.write( sLine )
    sLine = 'sDirectory_python=' + sDirectory_python + '\n'
    pFile.write( sLine )
    sLine = 'sBasename_python=' + sBasename_python + '\n'
    pFile.write( sLine )
    sLine = 'sFilename_python=' + '$sDirectory_python' + "/" + '$sBasename_python' + '\n'
    pFile.write( sLine )

    sLine = 'echo " Job " ' + '${SLURM_JOBID}' + ' is launched' + '\n'
    pFile.write( sLine )

    sLine = 'conda deactivate' + '\n'
    pFile.write( sLine )

    if iFlag_resubmit ==1: #advance job for checkpoint
        #now prepare the resubmit part
        sLine = 'iIndex="$(sed -n ' + ' 1p ' + sBasename_checkpoint + ' | xargs)"' + '\n'
        pFile.write( sLine )
        sLine = 'iIndex="$(sed -n ' + ' 2p ' + sBasename_checkpoint + ' | xargs)"' + '\n'
        pFile.write( sLine )
        sLine = 'iIndex="$(sed -n ' + ' 3p ' + sBasename_checkpoint + ' | xargs)"' + '\n'
        pFile.write( sLine )

        sLine = 'if (($iIndex == 0));then' + '\n'
        pFile.write( sLine )
        sLine = '    echo "All tasks are finished"'      + '\n'
        pFile.write( sLine )
        sLine = 'else'      + '\n'
        pFile.write( sLine )
        sLine = '    echo "Job $SLURM_JOBID at $(date) will be re-submitted with new indices:, $sStart, $sEnd"'        + '\n'
        pFile.write( sLine )
        sLine = '    sBasename_job="resubmit_${iIndex}.job"'      + '\n'
        pFile.write( sLine )

        sLine = '    sbatch $sDirectory_job/$sBasename_job'      + '\n'
        pFile.write( sLine )
        sLine = 'fi'      + '\n'
        pFile.write( sLine )

    else:
        pass

    sLine = 'echo "Finished"'      + '\n'
    pFile.write( sLine )
    pFile.close()

    return



import os, sys
from netCDF4 import Dataset
def replace_variable_in_netcdf(sFilename_old, sFilename_new, aData_in, sVariable_in):

    if os.path.exists(sFilename_old):
        print("Yep, I can read that file!")
    else:
        print("Nope, the path doesn't reach your file. Go research filepath in python")
        exit
    pDatasets_in = Dataset(sFilename_old)
    netcdf_format = pDatasets_in.file_format
    #output file
    pDatasets_out = Dataset(sFilename_new, "w", format=netcdf_format)
    for sKey, iValue in pDatasets_in.dimensions.items():
        dummy = len(iValue)
        if not iValue.isunlimited():            
            pDatasets_out.createDimension(sKey, dummy)            
        else:
            pDatasets_out.createDimension(sKey, dummy )

    # Copy variables
    for sKey, aValue in pDatasets_in.variables.items():        
        print(aValue.datatype)
        print( aValue.dimensions)
        # we need to take care of rec dimension
        dummy = aValue.dimensions
        if(sKey != sVariable_in): #only copy other dataset first
            outVar = pDatasets_out.createVariable(sKey, aValue.datatype, dummy )        
            for sAttribute in aValue.ncattrs():            
                outVar.setncatts( { sAttribute: aValue.getncattr(sAttribute) } )

            outVar[:] = aValue[:]
        else:
            pDataType = aValue.datatype
            pDimension = aValue.dimensions
            pUnit = aValue.units

            pass
        # close the output file
    #replace variable 
    pVar3 = pDatasets_out.createVariable(sVariable_in, pDataType, pDimension  ) 
    pVar3[:] = aData_in
    pVar3.description = sVariable_in
    pVar3.units = pUnit
    pDatasets_out.close()
    return


import os

def prepare_parafly_slurm_job_script(sBasename_job,  sBasename_parafly,   sDirectory_job, sEmail, iWalltime_in = None, nNode_in = None, nThread_in=None, sJob_name_in =None,  sPython_env_in =None, sQueue_in=None):
    """
    prepare a job script for parafly

    Args:
        sBasename_job (string): The base job name
        sBasename_parafly (string): The parafly script filename
        sDirectory_job (string): The path where parafly runs the script
        sEmail (string): Email address for job notification
        iWalltime_in (int, optional): Walltime in hour. Defaults to None.
        nNode_in (int, optional): NUmber of node. Defaults to None.
        nThread_in (int, optional): Number of node. Defaults to None.
        sJob_name_in (string, optional): Job name. Defaults to None.
        sPython_env_in (string, optional): Conda python environment. Defaults to None.
        sQueue_in (string, optional): HPC queue or partition. Defaults to None.
    """
    if iWalltime_in is not None:
        iWalltime = iWalltime_in            
    else:
        iWalltime = 2
    if nNode_in is not None:
        iNode = nNode_in            
    else:
        iNode = 1
    if nThread_in is not None:
        nThread = nThread_in            
    else:
        nThread = 40
   
    if sJob_name_in is not None:
        sJob_name = sJob_name_in            
    else:
        sJob_name = 'parafly'
    if sPython_env_in is not None:
        sPython_env = sPython_env_in            
    else:
        sPython_env = 'base'
        
    if sQueue_in is not None:
        sQueue = sQueue_in            
    else:
        sQueue = 'short'
        
    sWalltime ="{:0d}".format(iWalltime  )
    sNode =  "{:0d}".format(iNode  )
    sThread =   "{:0d}".format(nThread  )
    
    os.chdir(sDirectory_job)
    
    ofs =  open(sBasename_job,"w")  #write mode 
    sLine = '#!/bin/bash' + '\n'
    ofs.write( sLine ) 
    sLine = '#SBATCH --account=e3sm' + '\n'
    ofs.write( sLine ) 

    #sLine = '#SBATCH --begin=now+1minutes' + '\n'
    #ofs.write( sLine ) 

    sLine = '#SBATCH --cpus-per-task=1 ' + '\n'
    ofs.write( sLine ) 

    sLine = '#SBATCH --dependency=singleton ' + '\n'
    ofs.write( sLine )
    sLine = '#SBATCH --error=stderr_%j.err' + '\n'
    ofs.write( sLine ) 
    sLine = '#SBATCH --job-name=' + sJob_name + '  # create a name for your job' + '\n'
    ofs.write( sLine ) 
    sLine = '#SBATCH --mail-type=ALL' + '\n'
    ofs.write( sLine ) 
    sLine = '#SBATCH --mail-user=' + sEmail + '\n'
    ofs.write( sLine ) 

    sLine = '#SBATCH --nodes=' + sNode + ' # node count' + '\n'
    ofs.write( sLine ) 
    sLine = '#SBATCH --ntasks=' + sThread + ' # total number of tasks' + '\n'
    ofs.write( sLine ) 
    sLine = '#SBATCH --output=stdout_%j.out' + '\n'
    ofs.write( sLine ) 

    sLine = '#SBATCH --partition=' + sQueue + '\n'  #can be improved here
    ofs.write( sLine ) 
    sLine = '#SBATCH --time=' + sWalltime +':00:00   # total run time limit (HH:MM:SS)' + '\n'
    ofs.write( sLine ) 

    sLine = 'module purge' + '\n'
    ofs.write( sLine ) 
    sLine = 'module load parafly/2013' + '\n'
    ofs.write( sLine ) 
    sLine = 'module load anaconda3/2019.03' + '\n'
    ofs.write( sLine ) 
    sLine = 'source /share/apps/anaconda3/2019.03/etc/profile.d/conda.sh' + '\n'
    ofs.write( sLine ) 
    sLine = 'unset PYTHONHOME' + '\n'
    ofs.write( sLine ) 
    sLine = 'conda activate ' + sPython_env + '\n'
    ofs.write( sLine )   

    sLine = 'ParaFly -c ' + sBasename_parafly + ' -CPU ' + sThread + ' -failed_cmds rerun.txt' + '\n'
    ofs.write( sLine ) 
    
    sLine = 'echo " Job " ' + '${SLURM_JOBID}' + ' is launched' + '\n'
    ofs.write( sLine ) 

    sLine = 'conda deactivate' + '\n'
    ofs.write( sLine ) 
    
    sLine = 'echo "Finished"'      + '\n'
    ofs.write( sLine ) 
    ofs.close() 
    
    return



def slurm_update_checkpoint_file(iIndex_restart,     iIndex_start,     iIndex_end,    sFilename_checkpoint):
    """
    Write and save the checkpoint information

    Args:
        iIndex_restart (int): [description]
        iIndex_start (int): [description]
        iIndex_end (int): [description]
        sFilename_checkpoint (string): [description]
    """
    ofs = open(sFilename_checkpoint, "w")  #write mode 
    sLine = "{:0d}".format( iIndex_restart ) + '\n'
    ofs.write( sLine ) 
    sLine = "{:0d}".format( iIndex_start )  + '\n'
    ofs.write( sLine ) 
    sLine = "{:0d}".format( iIndex_end )  + '\n'
    ofs.write( sLine ) 
    ofs.close() 
    
        
    return


import os, sys
from pyearth.system.define_global_variables import *

def slurm_prepare_job_script_python(iIndex_start, iIndex_end,      sBasename_job,     sDirectory_job,     sFilename_python,       iWalltime_in = None,     nNode_in = None,     nThread_in = None,   sAccount =None,    sEmail=None,      sQueue_in = None):
    """
    Prepare a slurm job script for python without checkpoint

    Args:
        iIndex_start ([type]): The start index
        iIndex_end ([type]): The end index
        sBasename_job ([type]): The basename of job
        sDirectory_job ([type]): [description]
        sFilename_python ([type]): [description]
        iWalltime_in ([type], optional): [description]. Defaults to None.
        nNode_in ([type], optional): [description]. Defaults to None.
        nThread_in ([type], optional): [description]. Defaults to None.
        sAccount ([type], optional): [description]. Defaults to None.
        sEmail ([type], optional): [description]. Defaults to None.
        sQueue_in ([type], optional): [description]. Defaults to None.
    """
    if nNode_in is not None:
        iNode = nNode_in
    else:
        iNode = 1
    if nThread_in is not None:
        nThread = nThread_in
    else:
        nThread = 24

    if sQueue_in is not None:
        sQueue = sQueue_in
    else:
        sQueue = 'short'
    if iWalltime_in is not None:
        iWalltime = iWalltime_in
    else:
        iWalltime = 2


    nTask = iIndex_end - iIndex_start + 1
    nChunkPerThread = nTask // nThread

    sNode =  "{:0d}".format(iNode  )
    sTask =   "{:0d}".format(nTask  )
    sWalltime ="{:0d}".format(iWalltime  )
    os.chdir(sDirectory_job)

    for iRank in range(1, nThread+1):
        if iRank == 1:
            iStart = (nThread-1) * nChunkPerThread + iIndex_start
            iEnd = iIndex_end
        else:
            iStart = (iRank-2) * nChunkPerThread + iIndex_start
            iEnd = (iRank-1) * nChunkPerThread + iIndex_start - 1
            sStart = "{:0d}".format(iStart  )
            sEnd =  "{:0d}".format(iEnd  )

        sFilename_job = sDirectory_job + slash + sBasename_job + "{:03d}".format(iRank) + '.job'
        pFile =  open(sFilename_job,"w")  #write mode
        sLine = '#!/bin/bash' + '\n'
        pFile.write( sLine )

        if sAccount is not None:
            sLine = '#SBATCH --account=' + sAccount  + '\n'
            pFile.write( sLine )

        sLine = '#SBATCH --begin=now+1minutes' + '\n'
        #pFile.write( sLine )

        sLine = '#SBATCH --cpus-per-task=1 ' + '\n'
        pFile.write( sLine )

        sLine = '#SBATCH --dependency=singleton ' + '\n'
        #pFile.write( sLine )
        sLine = '#SBATCH --error=stderr_%j.err' + '\n'
        pFile.write( sLine )
        sLine = '#SBATCH --job-name=' \
            + sBasename_job + "{:03d}".format(iRank) + '  # create a name for your job' + '\n'
        pFile.write( sLine )
        sLine = '#SBATCH --mail-type=ALL' + '\n'
        pFile.write( sLine )

        if sEmail is not None:
            sLine = '#SBATCH --mail-user=' + sEmail + '\n'
            pFile.write( sLine )

        sLine = '#SBATCH --nodes=' + sNode + ' # node count' + '\n'
        pFile.write( sLine )
        sLine = '#SBATCH --ntasks=1'  + ' # total number of tasks' + '\n'
        pFile.write( sLine )
        sLine = '#SBATCH --output=stdout_%j.out' + '\n'
        pFile.write( sLine )

        sLine = '#SBATCH --partition=' + sQueue + '\n'  #can be improved here
        pFile.write( sLine )

        sLine = '#SBATCH --time=' + sWalltime +':00:00      # total run time limit (HH:MM:SS)' + '\n'
        pFile.write( sLine )

        sLine = 'module purge' + '\n'
        pFile.write( sLine )
        sLine = 'module load anaconda3/2019.03' + '\n'
        pFile.write( sLine )
        sLine = 'source /share/apps/anaconda3/2019.03/etc/profile.d/conda.sh' + '\n'
        pFile.write( sLine )
        sLine = 'unset PYTHONHOME' + '\n'
        pFile.write( sLine )
        sLine = 'conda activate gdalenv' + '\n'
        pFile.write( sLine )

        sLine = 'python ' + sFilename_python \
            + ' --iIndex_start ' + sStart \
            + ' --iIndex_end '  + sEnd + '\n'
        pFile.write( sLine )

        sLine = 'echo " Job " ' + '${SLURM_JOBID}' + ' is launched' + '\n'
        pFile.write( sLine )

        sLine = 'conda deactivate' + '\n'
        pFile.write( sLine )
        sLine = 'echo "Finished"'      + '\n'
        pFile.write( sLine )
        pFile.close()

    return



import os,sys

from pyearth.system.define_global_variables import *

def prepare_parafly_python_command_file(iIndex_start,     iIndex_end,    nThread,     sFilename_parafly,     sFilename_python):
    """
    Prepare a parafly file which runs python script

    Args:
        iIndex_start (int): The start index
        iIndex_end (int): The end index
        nThread (int): Number of thread
        sFilename_parafly (string): The python binary
        sFilename_python (string): The parafly filename
    """
    
    ofs =  open(sFilename_parafly,"w")  #write mode 
    nTask = iIndex_end - iIndex_start + 1
    nChunkPerThread = nTask // nThread 
    
    for iRank in range(nThread):
        if iRank == 0:
            iStart = (nThread-1) * nChunkPerThread + iIndex_start
            iEnd = iIndex_end      
        else:
            iStart = (iRank-1) * nChunkPerThread + iIndex_start
            iEnd = (iRank) * nChunkPerThread + iIndex_start - 1
            
        sStart = "{:0d}".format( iStart ) 
        sEnd = "{:0d}".format( iEnd ) 
        sLine = 'python ' + sFilename_python \
            + ' --iIndex_start ' + sStart \
            + ' --iIndex_end '  + sEnd + '\n'
       
        ofs.write(sLine) 
    
    ofs.close()
    
    return




import numpy as np
import statistics 
from numpy  import array
from pyearth.system.define_global_variables import *

def search_neighbors(iIndex_in, aArray_in, iWindow_size_in = None):
    """
    Gap filling a 2D array

    Args:
        iIndex_in (int): [description]
        aArray_in (numpy.array): Input data
        iWindow_size_in (int, optional): Search window. Defaults to None.

    Returns:
        numpy.array: Gap filled array
    """
    missing_value = np.nan
    if iWindow_size_in is not None:
        iWindow_size = iWindow_size_in           
    else:
        iWindow_size = 1

    iLength =  len(aArray_in)
    iStart = iWindow_size
    iEnd = iLength - iWindow_size
    iLength_out = iWindow_size * 2 + 1
    aArray_out = np.full( iLength_out , np.nan , dtype = float )    
    if iIndex_in >= iStart and iIndex_in < iEnd  :
        aArray_out=  aArray_in[ (iIndex_in - iWindow_size) : (iIndex_in + iWindow_size+1)   ]     
 
    return aArray_out


def gap_fill_by_window (aArray_in, iWindow_size_in = None):
    """
    Gap filling a 2D array

    Args:
        aArray_in (numpy.array): 2D array
        iWindow_size_in (int, optional): search window. Defaults to None.

    Returns:
        numpy.array: Gap filled 2D array
    """
    iLength = len(aArray_in)
    iWindow_size = iWindow_size_in
    aArray_out = aArray_in

    iStart = iWindow_size
    iEnd = iLength - iWindow_size
   
    for iIndex in range( iStart , iEnd  ):
        dummy = aArray_in[iIndex]
        if ( np.isnan(dummy) )  :
            aNeighbors = search_neighbors(iIndex, aArray_out, iWindow_size_in = iWindow_size)
            
            aNeighbors = aNeighbors.reshape( len(aNeighbors) )
            nan_index = np.where( np.isnan(aNeighbors) == True)

            nan_count  = len(nan_index[0])
             #only if half of the aNeighbors are qualified till we use this
             #gap fill
            if nan_count <= (iWindow_size +1):
               #aNeighbors=gap_fill(aNeighbors,/spline)

                good_index = np.where( np.isnan(aNeighbors) == False)

                aNeighbors_good = aNeighbors[ good_index ]
                good_count = len(aNeighbors_good)
                
                if good_count <  10:
                    aArray_out[iIndex] = statistics.median(aNeighbors_good)
                    pass
                   
                else:
                    aArray_out[iIndex] = statistics.median(aNeighbors_good)
                    pass
                
                

            else:
                
                pass
             #good elements count is not enough

            
        else:
   
            pass
     
        

    return aArray_out